import os
from aloft import chart
from unittest import TestCase
from unittest.mock import call, patch


class TestApplyCharts(TestCase):

    @patch('aloft.chart.volume.restore_volumes')
    @patch('aloft.chart.k8s.create_namespace')
    @patch('aloft.chart.execute')
    @patch('aloft.chart.chart_config.generate_value_files')
    @patch('aloft.chart.aws.get_secret')
    @patch('aloft.chart.volume.lock_volumes')
    @patch('aloft.chart.output.print_action')
    def test_should_apply_charts(self,
                                 mock_print_action,
                                 mock_lock_volumes,
                                 mock_get_secret,
                                 mock_generate_value_files,
                                 mock_execute,
                                 mock_create_namespace,
                                 mock_restore_volumes):

        os.environ['ALOFT_CONFIG'] = f'test-config'
        mock_execute.return_value = ''
        mock_generate_value_files.return_value = ['TEST_VALUE_FILE_1', 'TEST_VALUE_FILE_2', 'TEST_VALUE_FILE_3']
        mock_get_secret.side_effect = [
            'TEST_GIT_USERNAME', 'TEST_GIT_PASSWORD', 'TEST_DOCKER_USERNAME', 'TEST_DOCKER_PASSWORD'
        ]

        chart.apply_charts('prod', 'project-tools', ['test-jenkins', 'test-nginx-ingress'], None)

        mock_execute.assert_has_calls([
            call('kubectl -n project-tools-test-namespace delete secret test-jenkins-secrets',
                 ['NotFound'], quiet=True),
            call('kubectl -n project-tools-test-namespace create secret generic test-jenkins-secrets'
                 '  --from-literal=git-username=TEST_GIT_USERNAME'
                 ' --from-literal=git-password=TEST_GIT_PASSWORD'
                 ' --from-literal=docker-username=TEST_DOCKER_USERNAME'
                 ' --from-literal=docker-password=TEST_DOCKER_PASSWORD', quiet=True),
            call('helm dependencies build test-config/charts/test-jenkins'),
            call('helm upgrade -i project-tools-test-namespace-test-jenkins'
                 ' --namespace project-tools-test-namespace test-config/charts/test-jenkins'
                 ' -f TEST_VALUE_FILE_1 -f TEST_VALUE_FILE_2 -f TEST_VALUE_FILE_3'),
            call('helm dependencies build test-config/charts/test-nginx-ingress'),
            call('helm upgrade -i project-tools-test-namespace-test-nginx-ingress'
                 ' --namespace project-tools-test-namespace test-config/charts/test-nginx-ingress'
                 ' -f TEST_VALUE_FILE_1 -f TEST_VALUE_FILE_2 -f TEST_VALUE_FILE_3')
        ])
        mock_create_namespace.assert_called_once_with('project-tools-test-namespace')
        mock_get_secret.assert_has_calls([
            call('project-tools.git-username'),
            call('project-tools.git-password'),
            call('project-tools.docker-username'),
            call('project-tools.docker-password')
        ])
        mock_restore_volumes.assert_called_once_with('prod',
                                                     'project-tools',
                                                     ['test-jenkins',
                                                      'test-nginx-ingress'],
                                                     None)
        mock_lock_volumes.assert_called_once_with('prod', 'project-tools', ['test-jenkins', 'test-nginx-ingress'], None)
        mock_print_action.assert_has_calls([
            call('Creating secret test-jenkins-secrets with keys'
                 ' [\'git-username\', \'git-password\', \'docker-username\', \'docker-password\']')
        ])


class TestDeleteCharts(TestCase):

    @patch('aloft.chart.execute')
    @patch('aloft.chart.k8s.delete_namespace_if_empty')
    @patch('aloft.chart.volume.remove_released_volume_resources')
    def test_should_delete_charts(self, mock_remove, mock_delete_namespace_if_empty, mock_execute):
        os.environ['ALOFT_CONFIG'] = f'test-config'

        chart.delete_charts('prod', 'project-tools', ['test-jenkins', 'test-nginx-ingress'], None)

        mock_execute.assert_has_calls([
            call('helm delete --purge project-tools-test-namespace-test-jenkins', ['not found']),
            call('kubectl -n project-tools-test-namespace delete secret test-jenkins-secrets', ['NotFound']),
            call('helm delete --purge project-tools-test-namespace-test-nginx-ingress', ['not found'])
        ])
        mock_delete_namespace_if_empty.assert_called_once_with('project-tools-test-namespace')
        mock_remove.assert_called_once_with('prod', 'project-tools', ['test-jenkins', 'test-nginx-ingress'], None)



from aloft import cluster
import os
from unittest import TestCase
from unittest.mock import call, patch


class TestCreateCluster(TestCase):

    @patch('aloft.cluster.get_or_create_vpc')
    @patch('aloft.cluster.execute')
    @patch('aloft.cluster.output.print_action')
    @patch('aloft.cluster.output.write_to_file')
    @patch('aloft.cluster.os.remove')
    @patch('aloft.cluster.aws.get_vpn_gateway_id')
    @patch('aloft.cluster.aws.get_route_table_ids')
    @patch('aloft.cluster.aws.propagate_route_table')
    @patch('aloft.cluster.aws.set_role_policy')
    @patch('aloft.cluster.time.sleep')
    @patch('aloft.cluster.aws.conditionally_create_hosted_zone_and_setup_ns_records')
    @patch('aloft.cluster.host.wait_for_domain_to_resolve')
    @patch('aloft.cluster.aws.get_or_create_s3_bucket')
    @patch.dict('aloft.cluster.os.environ', {'KUBECONFIG': 'TEST_ORIGINAL_KUBECONFIG'})
    def test_should_create_cluster(self,
                                   mock_get_or_create_s3_bucket,
                                   mock_wait_for_domain_to_resolve,
                                   mock_conditionally_create_hosted_zone_and_setup_ns_records,
                                   mock_sleep,
                                   mock_set_role_policy,
                                   mock_propagate_route_table,
                                   mock_get_route_table_ids,
                                   mock_get_vpn_gateway_id,
                                   mock_remove,
                                   mock_write_to_file,
                                   mock_print_action,
                                   mock_execute,
                                   mock_get_or_create_vpc):

        os.environ['ALOFT_CONFIG'] = f'test-config'
        mock_get_or_create_vpc.return_value = {'Name': 'sandbox', 'VpcId': 'TEST_AWS_VPC_ID'}
        mock_get_vpn_gateway_id.return_value = 'TEST_GATEWAY_ID'
        mock_get_route_table_ids.return_value = ['TEST_ROUTE_TABLE_ID_1', 'TEST_ROUTE_TABLE_ID_2']

        cluster.conditionally_create_vpc_and_cluster('cl1.sandbox.test.com')

        mock_get_or_create_vpc.assert_called_once_with('sandbox.test.com')
        mock_get_or_create_s3_bucket.assert_called_once_with('k8s-state.sandbox.test.com')
        mock_execute.assert_has_calls([
            call('kops toolbox template'
                 ' --values test-config/clusters/template/values.yaml'
                 ' --values test-config/clusters/values/test.com/sandbox/cl1/cluster.yaml'
                 ' --values test-config/clusters/values/test.com/sandbox/vpc.yaml'
                 ' --values /tmp/cl1.sandbox.test.com-dynamic.yaml'
                 ' --template test-config/clusters/template/template.yaml'
                 ' --format-yaml=true'
                 ' > /tmp/cl1.sandbox.test.com.yaml'),
            call('kops create -f /tmp/cl1.sandbox.test.com.yaml --state s3://k8s-state.sandbox.test.com'),
            call('ssh-keygen -q -f /tmp/cluster_rsa -t rsa -N ""'),
            call('kops create secret --name cl1.sandbox.test.com sshpublickey admin -i "/tmp/cluster_rsa.pub"'
                 ' --state s3://k8s-state.sandbox.test.com'),
            call('kops update cluster cl1.sandbox.test.com --yes --state s3://k8s-state.sandbox.test.com'),
            call('kops validate cluster cl1.sandbox.test.com --state s3://k8s-state.sandbox.test.com'),
            call('helm init'),
            call('kubectl rollout status -w deployment/tiller-deploy --namespace=kube-system')
        ])
        mock_write_to_file.assert_called_once_with(
            '/tmp/cl1.sandbox.test.com-dynamic.yaml',
            'networkId: TEST_AWS_VPC_ID\n'
            'clusterFQDN: cl1.sandbox.test.com\n'
            'kopsStateStore: s3://k8s-state.sandbox.test.com\n')
        mock_remove.assert_has_calls([
            call('/tmp/cluster_rsa'),
            call('/tmp/cluster_rsa.pub'),
            call('/tmp/cluster_rsa'),
            call('/tmp/cluster_rsa.pub')
        ])
        mock_propagate_route_table.assert_has_calls([
            call('TEST_ROUTE_TABLE_ID_1', 'TEST_GATEWAY_ID'),
            call('TEST_ROUTE_TABLE_ID_2', 'TEST_GATEWAY_ID')]
        )
        mock_set_role_policy.assert_called_once_with(
            'nodes.cl1.sandbox.test.com',
            'systemPolicy',
            '{\n  "Version": "2012-10-17",\n  "Statement": ["TEST_STATEMENT"]\n}'
        )

        mock_sleep.assert_called_once_with(300)
        mock_conditionally_create_hosted_zone_and_setup_ns_records.assert_called_once_with(
            'cl1.sandbox.test.com', 'sandbox.test.com'
        )
        mock_wait_for_domain_to_resolve.assert_called_once_with('cl1.sandbox.test.com')
        mock_print_action.assert_has_calls([
            call('Creating cluster cl1.sandbox.test.com in sandbox vpc.'),
            call('Waiting for cl1.sandbox.test.com to be ready.')
        ])


class TestDeleteCluster(TestCase):

    @patch('aloft.cluster.execute')
    @patch('aloft.cluster.aws.assume_role')
    def test_should_delete_cluster(self, mock_assume_role, mock_execute):
        cluster.delete_cluster('cl1.sandbox.test.com')

        mock_execute.assert_called_once_with(
            'kops delete cluster cl1.sandbox.test.com --yes --state s3://k8s-state.sandbox.test.com'
        )
        mock_assume_role.assert_called_once_with('sandbox')


class TestUseCluster(TestCase):

    @patch('aloft.cluster.execute')
    @patch('aloft.cluster.aws.assume_role')
    def test_should_use_cluster(self, mock_assume_role, mock_execute):
        cluster.use_cluster('cl1.sandbox.test.com')

        mock_execute.assert_called_once_with(
            'kops export kubecfg cl1.sandbox.test.com --state s3://k8s-state.sandbox.test.com'
        )
        mock_assume_role.assert_called_once_with('sandbox')


class TestGetClusters(TestCase):

    @patch('aloft.cluster.execute')
    @patch('aloft.cluster.print')
    @patch('aloft.cluster.aws.assume_role')
    def test_should_get_clusters_and_print_name_only(self, mock_assume_role, mock_print, mock_execute):
        os.environ['ALOFT_CONFIG'] = f'test-config'
        with open(f'test-data/kops-get-cluster-single-cluster.txt', 'r') as cluster_data_stream:
            mock_execute.return_value = cluster_data_stream.read()

        cluster.get_and_print_clusters('test.com', 'name')

        mock_assume_role.assert_called_once_with('sandbox')
        mock_execute.assert_called_once_with('kops get clusters -o yaml '
                                             '--state s3://k8s-state.sandbox.test.com', ['no clusters found'],
                                             quiet=True)
        mock_print.assert_has_calls([
            call('cl1.sandbox.test.com')
        ])

    @patch('aloft.cluster.execute')
    @patch('aloft.cluster.print')
    @patch('aloft.cluster.aws.assume_role')
    def test_should_get_clusters_and_print_name_only_for_multiple_clusters(self,
                                                                           mock_assume_role,
                                                                           mock_print,
                                                                           mock_execute):

        os.environ['ALOFT_CONFIG'] = f'test-config'
        with open(f'test-data/kops-get-cluster-multiple-clusters.txt', 'r') as cluster_data_stream:
            mock_execute.return_value = cluster_data_stream.read()

        cluster.get_and_print_clusters('test.com', 'name')

        mock_assume_role.assert_called_once_with('sandbox')
        mock_execute.assert_called_once_with(
            'kops get clusters -o yaml '
            '--state s3://k8s-state.sandbox.test.com', ['no clusters found'],
            quiet=True)
        mock_print.assert_has_calls([
            call('cl1.sandbox.test.com'),
            call('cl2.sandbox.test.com')
        ])

    @patch('aloft.cluster.execute')
    @patch('aloft.cluster.output.print_table')
    @patch('aloft.cluster.aws.assume_role')
    def test_should_get_clusters_and_print_text_for_multiple_clusters(self,
                                                                      mock_assume_role,
                                                                      mock_print_table,
                                                                      mock_execute):

        os.environ['ALOFT_CONFIG'] = f'test-config'
        with open(f'test-data/kops-get-cluster-multiple-clusters.txt', 'r') as cluster_data_stream:
            mock_execute.return_value = cluster_data_stream.read()

        cluster.get_and_print_clusters('test.com', 'text')

        mock_assume_role.assert_called_once_with('sandbox')
        mock_execute.assert_called_once_with(
            'kops get clusters -o yaml '
            '--state s3://k8s-state.sandbox.test.com', ['no clusters found'],
            quiet=True)
        mock_print_table.assert_has_calls([
            call(['NAME', 'VERSION', 'CIDR'], [
                ['cl1.sandbox.test.com', '1.9.1', '10.10.0.0/21'],
                ['cl2.sandbox.test.com', '1.8.5', '10.10.2.0/21']
            ], '{0: <40} {1: <10} {2: <18}')
        ])

    @patch('aloft.cluster.execute')
    @patch('aloft.cluster.output.print_list_as_yaml')
    @patch('aloft.cluster.aws.assume_role')
    def test_should_get_clusters_and_print_yaml_for_multiple_clusters(self,
                                                                      mock_assume_role,
                                                                      mock_print_list_as_yaml,
                                                                      mock_execute):

        os.environ['ALOFT_CONFIG'] = f'test-config'
        with open(f'test-data/kops-get-cluster-multiple-clusters.txt', 'r') as cluster_data_stream:
            mock_execute.return_value = cluster_data_stream.read()

        cluster.get_and_print_clusters('test.com', 'yaml')

        mock_assume_role.assert_called_once_with('sandbox')
        mock_execute.assert_called_once_with(
            'kops get clusters -o yaml '
            '--state s3://k8s-state.sandbox.test.com', ['no clusters found'],
            quiet=True)
        self.assertEqual(2, len(mock_print_list_as_yaml.call_args[0][0]))
        self.assertEqual('cl1.sandbox.test.com', mock_print_list_as_yaml.call_args[0][0][0]['metadata']['name'])
        self.assertEqual('cl2.sandbox.test.com', mock_print_list_as_yaml.call_args[0][0][1]['metadata']['name'])


class TestGetCurrentCluster(TestCase):

    @patch('aloft.cluster.execute')
    @patch('aloft.cluster.print')
    @patch('aloft.cluster.aws.assume_role')
    def test_should_get_current_cluster_and_print_name_only(self, mock_assume_role, mock_print, mock_execute):
        os.environ['ALOFT_CONFIG'] = f'test-config'
        with open(f'test-data/kops-get-cluster-multiple-clusters.txt', 'r') as cluster_data_stream:
            mock_execute.side_effect = ['cl2.sandbox.test.com\n', cluster_data_stream.read()]

        cluster.get_and_print_current_cluster('name')

        mock_assume_role.assert_called_once_with('sandbox')
        mock_execute.assert_has_calls([
            call('kubectl config current-context', quiet=True),
            call('kops get clusters -o yaml '
                 '--state s3://k8s-state.sandbox.test.com', ['no clusters found'],
                 quiet=True)
        ])
        mock_print.assert_has_calls([
            call('cl2.sandbox.test.com')
        ])

    @patch('aloft.cluster.execute')
    @patch('aloft.cluster.output.print_table')
    @patch('aloft.cluster.aws.assume_role')
    def test_should_get_current_cluster_and_print_text(self, mock_assume_role, mock_print_table, mock_execute):
        os.environ['ALOFT_CONFIG'] = f'test-config'
        with open(f'test-data/kops-get-cluster-multiple-clusters.txt', 'r') as cluster_data_stream:
            mock_execute.side_effect = ['cl2.sandbox.test.com\n', cluster_data_stream.read()]

        cluster.get_and_print_current_cluster('text')

        mock_assume_role.assert_called_once_with('sandbox')
        mock_execute.assert_has_calls([
            call('kubectl config current-context', quiet=True),
            call('kops get clusters -o yaml '
                 '--state s3://k8s-state.sandbox.test.com', ['no clusters found'],
                 quiet=True)
        ])
        mock_print_table.assert_has_calls([
            call(['NAME', 'VERSION', 'CIDR'], [
                ['cl2.sandbox.test.com', '1.8.5', '10.10.2.0/21']
            ], '{0: <40} {1: <10} {2: <18}')
        ])

    @patch('aloft.cluster.execute')
    @patch('aloft.cluster.output.print_list_as_yaml')
    @patch('aloft.cluster.aws.assume_role')
    def test_should_get_current_cluster_and_print_yaml(self,
                                                       mock_assume_role,
                                                       mock_print_list_as_yaml,
                                                       mock_execute):

        os.environ['ALOFT_CONFIG'] = f'test-config'
        with open(f'test-data/kops-get-cluster-multiple-clusters.txt', 'r') as cluster_data_stream:
            mock_execute.side_effect = ['cl2.sandbox.test.com\n', cluster_data_stream.read()]

        cluster.get_and_print_current_cluster('yaml')

        mock_assume_role.assert_called_once_with('sandbox')
        mock_execute.assert_has_calls([
            call('kubectl config current-context', quiet=True),
            call('kops get clusters -o yaml '
                 '--state s3://k8s-state.sandbox.test.com', ['no clusters found'],
                 quiet=True)
        ])
        self.assertEqual(1, len(mock_print_list_as_yaml.call_args[0][0]))
        self.assertEqual('cl2.sandbox.test.com', mock_print_list_as_yaml.call_args[0][0][0]['metadata']['name'])



import os

from aloft.process import execute
from aloft import aws
from aloft import k8s
from aloft import chart_config
from aloft import output
from aloft import volume


def apply_charts(release_id, chart_set, charts, sandbox_name=None):
    release_config = chart_config.get_release_config(chart_set, release_id, sandbox_name)
    namespace = release_config['namespace']
    k8s.create_namespace(namespace)

    volume.restore_volumes(release_id, chart_set, charts, sandbox_name)

    for chart in charts:
        value_filenames = chart_config.generate_value_files(release_id, chart_set, chart, release_config)
        secrets_config = chart_config.get_secrets_config_for_chart(chart_set, chart)
        apply_chart(chart, namespace, value_filenames, secrets_config)

    if release_config.get('lockVolumes', False):
        volume.lock_volumes(release_id, chart_set, charts, sandbox_name)


def apply_chart(chart, namespace, value_filenames, secrets_config):
    base_chart_config_directory = chart_config.get_base_chart_config_directory()
    chart_directory = f'{base_chart_config_directory}/charts/{chart}'
    value_args = ' '.join(map(lambda filename: f'-f {filename}', value_filenames))
    release_name = get_release_name(chart, namespace)

    create_secrets(chart, secrets_config, namespace)
    execute_install_hook('pre-install', chart_directory)
    execute(f'helm dependencies build {chart_directory}')
    execute(f'helm upgrade -i {release_name} --namespace {namespace} {chart_directory} {value_args}')
    execute_install_hook('post-install', chart_directory)


def delete_charts(release_id, chart_set, charts, sandbox_name):
    release_config = chart_config.get_release_config(chart_set, release_id, sandbox_name)
    namespace = release_config['namespace']

    for chart in charts:
        secrets_config = chart_config.get_secrets_config_for_chart(chart_set, chart)
        delete_chart(chart, namespace, secrets_config)

    k8s.delete_namespace_if_empty(namespace)

    volume.remove_released_volume_resources(release_id, chart_set, charts, sandbox_name)


def delete_chart(chart, namespace, secrets_config):
    base_chart_config_directory = chart_config.get_base_chart_config_directory()
    chart_directory = f'{base_chart_config_directory}/charts/{chart}'
    release_name = get_release_name(chart, namespace)

    execute_install_hook('pre-uninstall', chart_directory)
    execute(f'helm delete --purge {release_name}', ['not found'])
    delete_secrets(chart, secrets_config, namespace)
    execute_install_hook('post-uninstall', chart_directory)


def get_release_name(chart, namespace):
    release_name = chart

    if chart != namespace:
        release_name = f'{namespace}-{chart}'

    return release_name


def create_secrets(chart, secrets_config, namespace):
    secret_args = ''
    secret_keys = []

    if secrets_config:
        for item in secrets_config.get('items', []):
            vault_key = item.get('vaultKey', None)
            secret_key = item.get('secretKey', None)

            if secret_key:
                secret_keys.append(secret_key)
                secret_value = aws.get_secret(vault_key)

                if secret_value:
                    secret_args = f'{secret_args} --from-literal={secret_key}={secret_value}'

        if secret_args:
            secrets_name = secrets_config.get('secretName', f'{chart}-secret')
            output.print_action(f'Creating secret {secrets_name} with keys {secret_keys}')
            execute(f'kubectl -n {namespace} delete secret {secrets_name}', ['NotFound'], quiet=True)
            execute(f'kubectl -n {namespace} create secret generic {secrets_name} {secret_args}', quiet=True)


def delete_secrets(chart, secrets_config, namespace):
    if secrets_config:
        secrets_name = secrets_config.get('secretName', f'{chart}-secret')
        execute(f'kubectl -n {namespace} delete secret {secrets_name}', ['NotFound'])


def execute_install_hook(hook_type, chart_directory):
    hook_script = f'{chart_directory}/hooks/{hook_type}'

    if os.path.exists(hook_script):
        execute(hook_script)



from aloft import output
from unittest import TestCase
from unittest.mock import call, patch


class TestPrintOutput(TestCase):

    @patch('aloft.output.print')
    def test_should_print_action(self, mock_print):
        output.print_action('TEST_ACTION')

        mock_print.assert_called_once_with('**** TEST_ACTION\n')

    @patch('aloft.output.print')
    def test_should_print_details(self, mock_print):
        output.print_details('TEST_DETAIL_LINE_1\nTEST_DETAIL_LINE_2\nTEST_DETAIL_LINE_3')

        mock_print.assert_has_calls([
            call('     TEST_DETAIL_LINE_1'),
            call('     TEST_DETAIL_LINE_2'),
            call('     TEST_DETAIL_LINE_3')
        ])

    @patch('aloft.output.print')
    def test_should_print_detail_line(self, mock_print):
        output.print_details_line('TEST_DETAIL_LINE_1')

        mock_print.assert_called_once_with('     TEST_DETAIL_LINE_1')


class TestPrintTable(TestCase):

    @patch('aloft.output.print')
    def test_should_print_table(self, mock_print):
        output.print_table(
            ['HEADING_A', 'HEADING_B', 'HEADING_C'],
            [
                ['VALUE_1_1', 'VALUE_1_2', 'VALUE_1_3'],
                ['VALUE_2_1', 'VALUE_2_2', 'VALUE_2_3'],
                ['VALUE_3_1', 'VALUE_3_2', 'VALUE_3_3'],
                ['VALUE_4_1', 'VALUE_4_2', 'VALUE_4_3']
            ], '{0: <40} {1: <10} {2: <18}')

        mock_print.assert_has_calls([
            call('HEADING_A                                HEADING_B  HEADING_C         '),
            call('VALUE_1_1                                VALUE_1_2  VALUE_1_3         '),
            call('VALUE_2_1                                VALUE_2_2  VALUE_2_3         '),
            call('VALUE_3_1                                VALUE_3_2  VALUE_3_3         '),
            call('VALUE_4_1                                VALUE_4_2  VALUE_4_3         '),
        ])


class TestPrintYamlList(TestCase):

    @patch('aloft.output.print')
    def test_should_print_list_as_yaml(self, mock_print):
        output.print_list_as_yaml([
            {'TEST_KEY_1': 'TEST_VALUE_1'},
            {'TEST_KEY_2': 'TEST_VALUE_2'},
            {'TEST_KEY_3': 'TEST_VALUE_3'}
        ])

        mock_print.assert_called_once_with('TEST_KEY_1: TEST_VALUE_1\n'
                                           '---\n'
                                           'TEST_KEY_2: TEST_VALUE_2\n'
                                           '---\n'
                                           'TEST_KEY_3: TEST_VALUE_3')



from aloft import aws
from unittest import TestCase
from unittest.mock import Mock, patch


class TestVpc(TestCase):

    @patch('aloft.aws.boto3.client')
    def test_should_get_vpc(self, mock_client):
        aws.credentials = {
            'AccessKeyId': 'TEST_ACCESS_KEY_ID',
            'SecretAccessKey': 'TEST_SECRET_ACCESS_KEY',
            'SessionToken': 'TEST_SESSION_TOKEN'
        }
        mock_ec2 = Mock()
        mock_ec2.describe_vpcs = Mock()
        mock_client.return_value = mock_ec2
        mock_ec2.describe_vpcs.return_value = {
            'Vpcs': [
                {
                    'CidrBlock': '10.10.0.0/21',
                    'DhcpOptionsId': 'dopt-c62433a1',
                    'State': 'available',
                    'VpcId': 'vpc-9669a4ef',
                    'InstanceTenancy': 'default',
                    'CidrBlockAssociationSet': [{
                        'AssociationId': 'vpc-cidr-assoc-8a7ef0e1',
                        'CidrBlock': '10.10.0.0/21',
                        'CidrBlockState': {'State': 'associated'}}],
                    'IsDefault': False,
                    'Tags': [
                        {
                            'Key': 'Name',
                            'Value': 'sandbox'
                        },
                        {
                            'Key': 'kubernetes.io/cluster/cl1.sandbox.test.com',
                            'Value': 'shared'
                        },
                        {
                            'Key': 'kubernetes.io/cluster/cl2.sandbox.test.com',
                            'Value': 'shared'
                        },
                        {
                            'Key': 'kubernetes.io/cluster/cl3.sandbox.test.com',
                            'Value': 'shared'
                        },
                        {
                            'Key': 'kubernetes.io/cluster/cl4.sandbox.test.com',
                            'Value': 'shared'
                        }
                    ]
                }
            ]
        }

        vpc = aws.get_vpc_by_name('sandbox')

        self.assertEqual(mock_ec2.describe_vpcs.return_value['Vpcs'][0], vpc)



from aloft import input
from unittest import TestCase
from unittest.mock import Mock, patch


class TestGetDirectoryFromEnvironment(TestCase):

    @patch('aloft.input.os')
    def test_should_return_directory_path_if_exists(self, mock_os):
        mock_os.environ = {'TEST_KEY': 'TEST_PATH'}
        mock_os.path.expanduser = Mock(return_value='TEST_EXPANDED_PATH')
        mock_os.path.isdir = Mock(return_value=True)

        directory = input.get_directory_from_environment('TEST_KEY')

        self.assertEqual('TEST_EXPANDED_PATH', directory)
        mock_os.path.expanduser.assert_called_once_with('TEST_PATH')

    @patch('aloft.input.os')
    def test_should_raise_if_not_a_directory(self, mock_os):
        mock_os.environ = {'TEST_KEY': 'TEST_PATH'}
        mock_os.path.expanduser = Mock(return_value='TEST_EXPANDED_PATH')
        mock_os.path.isdir = Mock(return_value=False)

        with self.assertRaises(NotADirectoryError) as cm:
            input.get_directory_from_environment('TEST_KEY')

        self.assertEqual('TEST_KEY must reference a directory: TEST_EXPANDED_PATH', str(cm.exception))

    @patch('aloft.input.os')
    def test_should_raise_if_environment_variable_not_found(self, mock_os):
        mock_os.environ = {'TEST_KEY': None}
        mock_os.path.expanduser = Mock(return_value=None)
        mock_os.path.isdir = Mock(return_value=False)

        with self.assertRaises(NotADirectoryError) as cm:
            input.get_directory_from_environment('TEST_KEY')

        self.assertEqual('TEST_KEY environment variable has not been set', str(cm.exception))


class TestRenderTemplateFile(TestCase):

    @patch('aloft.input.jinja2', return_value=Mock())
    def test_should_render_template(self, mock_jinja2):
        mock_jinja2.render = Mock(return_value='TEST_RENDERED_CONTENT')
        mock_jinja2.get_template = Mock(return_value=mock_jinja2)
        mock_jinja2.Environment = Mock(return_value=mock_jinja2)
        mock_jinja2.FileSystemLoader = Mock(return_value='TEST_LOADER')

        context = {
            'TEST_KEY_1': 'TEST_VALUE_1',
            'TEST_KEY_2': 'TEST_VALUE_2',
            'TEST_KEY_3': 'TEST_VALUE_3'
        }
        output = input.render_template_file('TEST_TEMPLATE_PATH/TEST_TEMPLATE_FILENAME', context)

        self.assertEqual('TEST_RENDERED_CONTENT', output)
        mock_jinja2.FileSystemLoader.assert_called_once_with('TEST_TEMPLATE_PATH')
        mock_jinja2.Environment.assert_called_once_with(loader='TEST_LOADER')
        mock_jinja2.get_template.assert_called_once_with('TEST_TEMPLATE_FILENAME')
        mock_jinja2.render.assert_called_once_with(context)



import os
from aloft import cluster_config
from unittest import TestCase


class TestClusterConfig(TestCase):

    def test_should_get_cluster_config_values(self):
        os.environ['ALOFT_CONFIG'] = f'test-config'

        cluster_values = cluster_config.get_cluster_config_values('cl1.sandbox.test.com')

        expected_config_values = {
            'cluster': {
                'spec': {
                    'subnets': {
                        'privateSubnets':
                            [{'cidr': '10.10.0.0/25',
                              'zone': 'us-west-2a'},
                             {'cidr': '10.10.0.128/25',
                              'zone': 'us-west-2b'},
                             {'cidr': '10.10.1.0/25',
                              'zone': 'us-west-2c'}],
                        'publicSubnets':
                            [{'cidr': '10.10.1.128/27',
                              'zone': 'us-west-2a'},
                             {'cidr': '10.10.1.160/27',
                              'zone': 'us-west-2b'},
                             {'cidr': '10.10.1.192/27',
                              'zone': 'us-west-2c'}]}}},
            'masters': {
                'spec': {
                    'image': '099720109477/ubuntu/images/hvm-ssd/ubuntu-xenial-16.04-amd64-server-20171121.1',
                    'machineType': 't2.large',
                    'maxSize': 1,
                    'minSize': 1,
                    'zones': ['us-west-2a', 'us-west-2b', 'us-west-2c']}},
            'nodes': {
                'spec': {
                    'image': '099720109477/ubuntu/images/hvm-ssd/ubuntu-xenial-16.04-amd64-server-20171121.1',
                    'machineType': 'm4.large',
                    'maxSize': 4,
                    'minSize': 1,
                    'zones': ['us-west-2a', 'us-west-2b', 'us-west-2c']}}}
        self.assertEqual(expected_config_values, cluster_values)

    def test_should_get_cluster_config_filename(self):
        os.environ['ALOFT_CONFIG'] = f'test-config'

        filename = cluster_config.get_cluster_values_filename('cl1.sandbox.test.com')

        self.assertEqual('test-config/clusters/values/test.com/sandbox/cl1/cluster.yaml', filename)



import os
from aloft.vpc import get_vpc, get_or_create_vpc, delete_vpc
from unittest import TestCase
from unittest.mock import patch


class TestGetVpc(TestCase):

    @patch('aloft.vpc.aws.get_vpc_by_name')
    def test_should_get_vpc(self, mock_get_vpc_by_name):
        mock_get_vpc_by_name.return_value = {'Name': 'sandbox'}

        vpc = get_vpc('sandbox.test.com')

        self.assertEqual({'Name': 'sandbox'}, vpc)


class TestCreateVpc(TestCase):

    @patch('aloft.vpc.aws.get_vpc_by_name')
    def test_should_return_vpc_if_exists(self, mock_get_vpc_by_name):
        mock_get_vpc_by_name.return_value = {'Name': 'sandbox'}

        vpc = get_or_create_vpc('sandbox.test.com')

        self.assertEqual({'Name': 'sandbox'}, vpc)

    @patch('aloft.vpc.aws.get_vpc_by_name')
    @patch('aloft.vpc.aws.create_vpc')
    @patch('aloft.vpc.aws.create_and_attach_internet_gateway')
    @patch('aloft.vpc.aws.create_and_attach_vpn_gateway')
    def test_should_create_vpc_if_not_exists(self,
                                             mock_create_and_attach_vpn_gateway,
                                             mock_create_and_attach_internet_gateway,
                                             mock_create_vpc,
                                             mock_get_vpc_by_name):
        os.environ['ALOFT_CONFIG'] = f'test-config'
        mock_get_vpc_by_name.return_value = None
        mock_create_vpc.return_value = {'Name': 'sandbox'}

        vpc = get_or_create_vpc('sandbox.test.com')

        self.assertEqual({'Name': 'sandbox'}, vpc)
        mock_create_vpc.assert_called_once_with('sandbox', '10.10.0.0/21')
        mock_create_and_attach_internet_gateway.assert_called_once_with({'Name': 'sandbox'})
        mock_create_and_attach_vpn_gateway.assert_called_once_with({'Name': 'sandbox'})


class TestDeleteVpc(TestCase):

    @patch('aloft.vpc.aws.delete_network_and_vpc')
    def test_should_delete_vpc(self, mock_delete_network_and_vpc):
        delete_vpc('sandbox.test.com')

        mock_delete_network_and_vpc.assert_called_once_with('sandbox')



import os
from aloft import volume
from unittest import TestCase
from unittest.mock import call, patch


class TestLockVolumes(TestCase):

    @patch('aloft.volume.k8s.get_resources')
    @patch('aloft.volume.k8s.get_resource_by_name')
    @patch('aloft.volume.aws.delete_ec2_tags')
    @patch('aloft.volume.aws.create_ec2_tags')
    @patch('aloft.volume.aws.save_content_to_s3')
    @patch('aloft.volume.output.print_action')
    @patch('aloft.volume.output.print_details')
    @patch('aloft.volume.execute')
    def test_should_lock_volumes(self,
                                 mock_execute,
                                 mock_print_details,
                                 mock_print_action,
                                 mock_save_content_to_s3,
                                 mock_create_ec2_tags,
                                 mock_delete_ec2_tags,
                                 mock_get_resource_by_name,
                                 mock_get_resources):

        os.environ['ALOFT_CONFIG'] = f'test-config'
        mock_get_resources.return_value = [{'spec': {'volumeName': 'TEST_VOLUME_ID_NAME_1'}}]
        mock_get_resource_by_name.return_value = {
            'apiVersion': 'TEST_API_VERSION',
            'kind': 'PersistentVolume',
            'metadata': {'name': 'TEST_VOLUME_ID_NAME'},
            'spec': {
                'persistentVolumeReclaimPolicy': 'Delete',
                'awsElasticBlockStore': {'volumeID': 'TEST_PREFIX/TEST_VOLUME_ID'},
                'claimRef': {
                    'apiVersion': 'TEST_CLAIM_VERSION',
                    'kind': 'TEST_KIND',
                    'name': 'TEST_NAME',
                    'namespace': 'TEST_NAMESPACE'
                }
            }
        }

        volume.lock_volumes('prod', 'project-tools', ['test-jenkins', 'test-nginx-ingress'], None)

        mock_get_resources.assert_has_calls([
            call('pvc', 'project-tools-test-namespace',
                 [{'key': 'release', 'value': 'project-tools-test-namespace-test-jenkins', 'operator': '='}]),
            call('pvc', 'project-tools-test-namespace',
                 [{'key': 'release', 'value': 'project-tools-test-namespace-test-nginx-ingress', 'operator': '='}])
        ])
        mock_execute.assert_called_once_with(
            'kubectl patch pv TEST_VOLUME_ID_NAME -p \'{"spec":{"persistentVolumeReclaimPolicy":"Retain"}}\''
        )
        mock_get_resource_by_name.assert_has_calls([
            call('pv', 'TEST_VOLUME_ID_NAME_1'),
            call('pv', 'TEST_VOLUME_ID_NAME_1')
        ])
        mock_delete_ec2_tags.assert_has_calls([
            call('TEST_VOLUME_ID',
                 ['kubernetes.io/cluster/cl1.tools.test.com', 'KubernetesCluster', 'LockedOnCluster']),
            call('TEST_VOLUME_ID',
                 ['kubernetes.io/cluster/cl1.tools.test.com', 'KubernetesCluster', 'LockedOnCluster'])
        ])
        mock_create_ec2_tags.assert_has_calls([
            call('TEST_VOLUME_ID', {'LockedOnCluster': 'cl1.tools.test.com'}),
            call('TEST_VOLUME_ID', {'LockedOnCluster': 'cl1.tools.test.com'})
        ])
        mock_save_content_to_s3.assert_has_calls([
            call('apiVersion: TEST_API_VERSION\n'
                 'kind: PersistentVolume\n'
                 'metadata:\n'
                 '  name: TEST_VOLUME_ID_NAME\n'
                 'spec:\n  awsElasticBlockStore:\n'
                 '    volumeID: TEST_PREFIX/TEST_VOLUME_ID\n'
                 '  claimRef:\n'
                 '    apiVersion: TEST_CLAIM_VERSION\n'
                 '    kind: TEST_KIND\n'
                 '    name: TEST_NAME\n'
                 '    namespace: TEST_NAMESPACE\n'
                 '  persistentVolumeReclaimPolicy: Retain',
                 'persistent-volumes.tools.test.com',
                 'prod_project-tools_test-jenkins.yaml'),
            call('apiVersion: TEST_API_VERSION\n'
                 'kind: PersistentVolume\n'
                 'metadata:\n'
                 '  name: TEST_VOLUME_ID_NAME\n'
                 'spec:\n'
                 '  awsElasticBlockStore:\n'
                 '    volumeID: TEST_PREFIX/TEST_VOLUME_ID\n'
                 '  claimRef:\n'
                 '    apiVersion: TEST_CLAIM_VERSION\n'
                 '    kind: TEST_KIND\n'
                 '    name: TEST_NAME\n'
                 '    namespace: TEST_NAMESPACE\n'
                 '  persistentVolumeReclaimPolicy: Retain',
                 'persistent-volumes.tools.test.com',
                 'prod_project-tools_test-nginx-ingress.yaml')
        ])
        mock_print_action.assert_has_calls([
            call("Locking persistent volumes: ['TEST_VOLUME_ID_NAME_1']"),
            call('Setting persistent volume reclaim policy for TEST_VOLUME_ID_NAME to Retain'),
            call('Releasing persistent volume TEST_VOLUME_ID_NAME from Kops.'),
            call('Saving persistent volume definition to '
                 's3://persistent-volumes.tools.test.com/prod_project-tools_test-jenkins.yaml'),
            call("Locking persistent volumes: ['TEST_VOLUME_ID_NAME_1']"),
            call('Setting persistent volume reclaim policy for TEST_VOLUME_ID_NAME to Retain'),
            call('Releasing persistent volume TEST_VOLUME_ID_NAME from Kops.'),
            call('Saving persistent volume definition to '
                 's3://persistent-volumes.tools.test.com/prod_project-tools_test-nginx-ingress.yaml')
        ])
        mock_print_details.assert_called()


class TestUnlockVolumes(TestCase):

    @patch('aloft.volume.k8s.get_resources')
    @patch('aloft.volume.k8s.get_resource_by_name')
    @patch('aloft.volume.aws.delete_ec2_tags')
    @patch('aloft.volume.aws.create_ec2_tags')
    @patch('aloft.volume.aws.delete_file_in_s3')
    @patch('aloft.volume.output.print_action')
    def test_should_lock_volumess(self,
                                  mock_print_action,
                                  mock_delete_file_in_s3,
                                  mock_create_ec2_tags,
                                  mock_delete_ec2_tags,
                                  mock_get_resource_by_name,
                                  mock_get_resources):

        os.environ['ALOFT_CONFIG'] = f'test-config'
        mock_get_resources.return_value = [{'spec': {'volumeName': 'TEST_VOLUME_ID_NAME_1'}}]
        mock_get_resource_by_name.return_value = {
            'apiVersion': 'TEST_API_VERSION',
            'kind': 'PersistentVolume',
            'metadata': {'name': 'TEST_VOLUME_ID_NAME'},
            'spec': {
                'persistentVolumeReclaimPolicy': 'Delete',
                'awsElasticBlockStore': {'volumeID': 'TEST_PREFIX/TEST_VOLUME_ID'},
                'claimRef': {
                    'apiVersion': 'TEST_CLAIM_VERSION',
                    'kind': 'TEST_KIND',
                    'name': 'TEST_NAME',
                    'namespace': 'TEST_NAMESPACE'
                }
            }
        }

        volume.unlock_volumes('prod', 'project-tools', ['test-jenkins', 'test-nginx-ingress'], None)

        mock_get_resources.assert_has_calls([
            call('pvc', 'project-tools-test-namespace',
                 [{'key': 'release', 'value': 'project-tools-test-namespace-test-jenkins', 'operator': '='}]),
            call('pvc', 'project-tools-test-namespace',
                 [{'key': 'release', 'value': 'project-tools-test-namespace-test-nginx-ingress', 'operator': '='}])
        ])
        mock_get_resource_by_name.assert_has_calls([
            call('pv', 'TEST_VOLUME_ID_NAME_1'),
            call('pv', 'TEST_VOLUME_ID_NAME_1')
        ])
        mock_delete_ec2_tags.assert_has_calls([
            call('TEST_VOLUME_ID',
                 ['kubernetes.io/cluster/cl1.tools.test.com', 'KubernetesCluster', 'LockedOnCluster']),
            call('TEST_VOLUME_ID',
                 ['kubernetes.io/cluster/cl1.tools.test.com', 'KubernetesCluster', 'LockedOnCluster'])
        ])
        mock_create_ec2_tags.assert_has_calls([
            call('TEST_VOLUME_ID',
                 {'kubernetes.io/cluster/cl1.tools.test.com': 'owned', 'KubernetesCluster': 'cl1.tools.test.com'}),
            call('TEST_VOLUME_ID',
                 {'kubernetes.io/cluster/cl1.tools.test.com': 'owned', 'KubernetesCluster': 'cl1.tools.test.com'})
        ])
        mock_delete_file_in_s3.assert_has_calls([
            call('persistent-volumes.tools.test.com', 'prod_project-tools_test-jenkins.yaml'),
            call('persistent-volumes.tools.test.com', 'prod_project-tools_test-nginx-ingress.yaml')
        ])
        mock_print_action.assert_has_calls([
            call("Unlocking persistent volumes: ['TEST_VOLUME_ID_NAME_1']"),
            call('Setting persistent volume reclaim policy for TEST_VOLUME_ID_NAME to Delete'),
            call('Attaching persistent volume TEST_VOLUME_ID_NAME to Kops.'),
            call('Deleting persistent volume definition from '
                 's3://persistent-volumes.tools.test.com/prod_project-tools_test-jenkins.yaml'),
            call("Unlocking persistent volumes: ['TEST_VOLUME_ID_NAME_1']"),
            call('Setting persistent volume reclaim policy for TEST_VOLUME_ID_NAME to Delete'),
            call('Attaching persistent volume TEST_VOLUME_ID_NAME to Kops.'),
            call('Deleting persistent volume definition from '
                 's3://persistent-volumes.tools.test.com/prod_project-tools_test-nginx-ingress.yaml')
        ])


class TestRestoreVolumes(TestCase):

    def test_should_restore_volume(self):
        pass


class TestRemoveReleasedVolumeResources(TestCase):

    def test_should_remove_released_volume_resources(self):
        pass



from aloft import k8s
from unittest import TestCase
from unittest.mock import call, patch


class TestApplyFile(TestCase):

    @patch('aloft.k8s.execute')
    def test_should_apply_file(self, mock_execute):
        k8s.apply_file('TEST_FILENAME')

        mock_execute.assert_called_once_with('kubectl apply -f TEST_FILENAME')


class TestCreateNamespace(TestCase):

    @patch('aloft.k8s.execute')
    def test_should_create_namespace(self, mock_execute):
        k8s.create_namespace('TEST_NAMESPACE')

        mock_execute.assert_called_once_with('kubectl create namespace TEST_NAMESPACE', ['AlreadyExists'])


class TestDeleteNamespace(TestCase):

    @patch('aloft.k8s.execute')
    def test_should_always_delete_namespace(self, mock_execute):
        k8s.delete_namespace('TEST_NAMESPACE')

        mock_execute.assert_called_once_with('kubectl delete namespace TEST_NAMESPACE', ['NotFound'])

    @patch('aloft.k8s.execute')
    def test_should_delete_namespace_if_empty(self, mock_execute):
        mock_execute.return_value = ''

        k8s.delete_namespace_if_empty('TEST_NAMESPACE')

        mock_execute.assert_has_calls([
            call('helm ls -q --namespace TEST_NAMESPACE'),
            call('kubectl delete namespace TEST_NAMESPACE', ['NotFound'])
        ])

    @patch('aloft.k8s.execute')
    def test_should_not_delete_namespace_if_not_empty(self, mock_execute):
        mock_execute.return_value = 'TEST_EXISTING_CHART_NAME'

        k8s.delete_namespace_if_empty('TEST_NAMESPACE')

        mock_execute.assert_called_once_with('helm ls -q --namespace TEST_NAMESPACE')


class TestGetResources(TestCase):

    @patch('aloft.k8s.execute')
    def test_should_get_resource_by_label_if_exists(self, mock_execute):
        mock_execute.return_value = ('items:\n'
                                     ' - ITEM_1:\n'
                                     '    KEY_1: VALUE1\n'
                                     ' - ITEM_2:\n'
                                     '    KEY_1: VALUE1\n')
        selectors = [{'key': 'TEST_KEY', 'value': 'TEST_VALUE', 'operator': '='}]

        resources = k8s.get_resources('TEST_RESOURCE_TYPE', 'TEST_NAME_SPACE', selectors)

        mock_execute.assert_called_once_with('kubectl get TEST_RESOURCE_TYPE -n TEST_NAME_SPACE '
                                             '-l TEST_KEY=TEST_VALUE -o yaml', ['NotFound'])
        self.assertEqual([{'ITEM_1': {'KEY_1': 'VALUE1'}}, {'ITEM_2': {'KEY_1': 'VALUE1'}}], resources)

    @patch('aloft.k8s.execute')
    def test_should_return_empty_list_if_none_found(self, mock_execute):
        mock_execute.return_value = 'items: []'

        resources = k8s.get_resources('TEST_RESOURCE_TYPE', 'TEST_NAME_SPACE', None)

        mock_execute.assert_called_once_with('kubectl get TEST_RESOURCE_TYPE -n TEST_NAME_SPACE -o yaml', ['NotFound'])
        self.assertEqual([], resources)

    @patch('aloft.k8s.execute')
    def test_should_return_resource_by_name(self, mock_execute):
        mock_execute.return_value = ('ITEM_1:\n'
                                     '  KEY_1: VALUE1\n')

        resources = k8s.get_resource_by_name('TEST_RESOURCE_TYPE', 'TEST_NAME', 'TEST_NAME_SPACE')

        mock_execute.assert_called_once_with('kubectl get TEST_RESOURCE_TYPE TEST_NAME -n TEST_NAME_SPACE -o yaml',
                                             ['NotFound'])
        self.assertEqual({'ITEM_1': {'KEY_1': 'VALUE1'}}, resources)

    @patch('aloft.k8s.execute')
    def test_should_return_none_if_not_found_by_name(self, mock_execute):
        mock_execute.return_value = 'items: []'

        resources = k8s.get_resource_by_name('TEST_RESOURCE_TYPE', 'TEST_NAME', 'TEST_NAME_SPACE')

        mock_execute.assert_called_once_with('kubectl get TEST_RESOURCE_TYPE TEST_NAME -n TEST_NAME_SPACE -o yaml',
                                             ['NotFound'])
        self.assertEqual(None, resources)

    @patch('aloft.k8s.execute')
    def test_should_return_resource_by_name_without_namespace(self, mock_execute):
        mock_execute.return_value = ('ITEM_1:\n'
                                     '  KEY_1: VALUE1\n')

        resources = k8s.get_resource_by_name('TEST_RESOURCE_TYPE', resource_name='TEST_NAME')

        mock_execute.assert_called_once_with('kubectl get TEST_RESOURCE_TYPE TEST_NAME -o yaml', ['NotFound'])
        self.assertEqual({'ITEM_1': {'KEY_1': 'VALUE1'}}, resources)


class TestUseNamespace(TestCase):

    @patch('aloft.k8s.execute')
    def test_should_use_namespace(self, mock_execute):
        k8s.use_namespace('TEST_NAMESPACE')

        mock_execute.assert_called_once_with('kubectl config set-context'
                                             ' $(kubectl config current-context)'
                                             ' --namespace=TEST_NAMESPACE')
